@@ -328,26 +328,22 @@ class SessionContext(Context):
         from holado_system.system.global_system import GlobalSystem
         from holado_helper.debug.memory.memory_profiler import MemoryProfiler
         from holado_test.test_config import TestConfig
-        from holado_report.report.report_manager import ReportManager
         
         log_prefix = f"[After scenario '{scenario.name}'] "
         
         with self.__multitask_lock:
-            # Log error on failing scenario
-            category_validation, status_validation, step_failed, step_number = ReportManager.get_current_scenario_status_information(scenario)
-            if status_validation != "Passed":
-                msg_list = []
-                category_str = f" => {category_validation}" if category_validation else ""
-                if step_failed is not None:
-                    msg_list.append(f"Scenario {status_validation}{category_str}:  [scenario in {scenario.filename} at l.{scenario.line} - step {step_number} (l.{step_failed.line})]")
-                else:
-                    msg_list.append(f"Scenario {status_validation}{category_str}:  [scenario in {scenario.filename} at l.{scenario.line} - step ? (missing step implementation ?)]")
-                step_error_message = ReportManager.get_step_error_message(step_failed)
-                if step_error_message:
-                    msg_list.append(step_error_message)
-                msg = "\n".join(msg_list)
-                logger.error(msg)
+            # End scenario
+            if not self.has_feature_context(is_reference=True):
+                raise TechnicalException(f"{log_prefix}No feature context is defined")
+            if not self.has_scenario_context(is_reference=True):
+                raise TechnicalException(f"{log_prefix}No scenario context is defined")
+            self.get_scenario_context().end()
             
+            # Process actions at scenario end
+            try:
+                self.after_scenario_end(scenario)
+            except:
+                logger.exception(f"{log_prefix}Error while processing actions after scenario end")
             
             # Process after scenario
             logger.info(f"{log_prefix}Begin")
@@ -359,13 +355,6 @@ class SessionContext(Context):
                 # MemoryProfiler.log_tracker_diff(name="scenarios objects", prefix="[After scenario] ", level=logging.INFO, logger_=logger)  # @UndefinedVariable
             GlobalSystem.log_resource_usage(prefix=log_prefix, level=logging.INFO, logger_=logger)
             
-            if not self.has_feature_context(is_reference=True):
-                raise TechnicalException(f"{log_prefix}No feature context is defined")
-            if not self.has_scenario_context(is_reference=True):
-                raise TechnicalException(f"{log_prefix}No scenario context is defined")
-            
-            self.get_scenario_context().end()
-            
             # Post processes
             logger.info(f"{log_prefix}Post processing...")
             self.get_scenario_context().scope_manager.reset_scope_level()
@@ -394,7 +383,23 @@ class SessionContext(Context):
             
             logger.info(f"{log_prefix}End")
             logger.info(f"Finished scenario [{scenario.name}]")
+    
+    def after_scenario_end(self, scenario):
+        from holado_report.report.report_manager import ReportManager
         
+        # Log error on failing scenario
+        category_validation, status_validation, step_failed, step_number, scenario_context, step_context = ReportManager.get_current_scenario_status_information(scenario)
+        has_failed = status_validation != "Passed"
+        if has_failed:
+            msg_list = []
+            category_str = f" => {category_validation}" if category_validation else ""
+            msg_list.append(f"Scenario {status_validation}{category_str}: {ReportManager.format_context_period(scenario_context)} {ReportManager.format_scenario_short_description(scenario)} - {ReportManager.format_step_short_description(step_failed, step_number, step_context=step_context, dt_ref=scenario_context.start_datetime, has_failed=has_failed)}")
+            step_error_message = ReportManager.get_step_error_message(step_failed)
+            if step_error_message:
+                msg_list.append(step_error_message)
+            msg = "\n".join(msg_list)
+            logger.error(msg)
+    
     def has_scenario_context(self, is_reference=None):
         return self.has_feature_context(is_reference=is_reference) and self.get_feature_context().has_scenario
     
@@ -447,10 +452,16 @@ class SessionContext(Context):
             except:
                 logger.exception(f"{log_prefix}Error while updating report before step")
     
-    def after_step(self, step):
+    def after_step(self, step, has_started=True):
+        """Process after step
+        @param step: step instance
+        @param has_started: if False, the step is added but without execution. 
+            It is usually True and before_step was called before, except for undefined and skipped steps.
+        """
         log_prefix = f"[After step '{step}'] "
         
         with self.__multitask_step_lock:
+            from holado_test.common.context.step_context import StepContext
             from holado_core.common.exceptions.technical_exception import TechnicalException
             
             if not self.has_feature_context(is_reference=None):
@@ -460,12 +471,30 @@ class SessionContext(Context):
             scenario_context = self.get_scenario_context()
             
             # Manage step context
-            step_context = scenario_context.scope_manager.get_step_context()
-            step_context.end()
-            if step_context.status is None:
-                step_context.status = step.status.name
-            if logger.isEnabledFor(logging.DEBUG):
-                logger.debug(f"{log_prefix}Ended step {step_context} in scenario {scenario_context}")
+            if has_started:
+                step_context = scenario_context.scope_manager.get_step_context()
+                step_context.end()
+                if step_context.status is None:
+                    step_context.status = step.status.name
+                if logger.isEnabledFor(logging.DEBUG):
+                    logger.debug(f"{log_prefix}Ended step {step_context} in scenario {scenario_context}")
+            else:
+                # Manage step context
+                step_context = StepContext(step, do_start=False)
+                scenario_context.scope_manager.set_step_context(step_context)
+                
+                # Update scenario context
+                step_level = scenario_context.scope_manager.scope_level("steps")
+                if step_level == 0:
+                    if logger.isEnabledFor(logging.DEBUG):
+                        logger.debug(f"{log_prefix}Add step {step_context} in scenario {scenario_context}")
+                    scenario_context.add_step(step_context)
+                
+                # Process before step in reports
+                try:
+                    self.report_manager.before_step(step_context, step, step_level)
+                except:
+                    logger.exception(f"{log_prefix}Error while updating report before step for unstarted step")
             
             # Report
             try:
