@@ -14,16 +14,11 @@
 from holado.common.context.session_context import SessionContext
 from holado_core.common.tools.tools import Tools
 import logging
-# from holado_report.report.builders.json_execution_historic_report_builder import JsonExecutionHistoricReportBuilder
-from holado_report.report.builders.detailed_scenario_failed_report_builder import DetailedScenarioFailedReportBuilder
-from holado_report.report.builders.summary_report_builder import SummaryReportBuilder
-from holado_report.report.builders.summary_scenario_failed_report_builder import SummaryScenarioFailedReportBuilder
-from holado_report.report.builders.short_scenario_failed_report_builder import ShortScenarioFailedReportBuilder
 from holado_report.report.reports.base_report import BaseReport
 from holado_scripting.common.tools.evaluate_parameters import EvaluateParameters
-from holado_report.report.builders.summary_scenario_report_builder import SummaryScenarioReportBuilder
-from holado_report.report.builders.failure_report_builder import FailureReportBuilder
-from holado_report.report.builders.summary_scenario_by_category_report_builder import SummaryScenarioByCategoryReportBuilder
+from holado_python.common.tools.datetime import DateTime, FORMAT_DATETIME_ISO, FORMAT_TIME_ISO
+from holado.common.handlers.undefined import default_value, undefined_argument
+from holado.holado_config import Config
 # from holado_core.scenario.scenario_duration_manager import ScenarioDurationManager
 
 logger = logging.getLogger(__name__)
@@ -58,17 +53,25 @@ class ReportManager(BaseReport):
         self.__multitask_manager = multitask_manager
         
     def initialize_reports(self):
+        from holado_report.report.builders.detailed_scenario_report_builder import DetailedScenarioReportBuilder
+        from holado_report.report.builders.summary_report_builder import SummaryReportBuilder
+        from holado_report.report.builders.summary_scenario_report_builder import SummaryScenarioReportBuilder
+        from holado_report.report.builders.short_scenario_report_builder import ShortScenarioReportBuilder
+        from holado_report.report.builders.failure_report_builder import FailureReportBuilder
+        from holado_report.report.builders.summary_scenario_by_category_report_builder import SummaryScenarioByCategoryReportBuilder
+        from holado_report.report.builders.summary_by_category_report_builder import SummaryByCategoryReportBuilder
+        
         # self.set_execution_historic()
         
         if self.has_report_path:
             # fn = self.get_path("execution_historic.json")
-            # self.add_report_builder(JsonExecutionHistoricReportBuilder(self.execution_historic, fn))
+            # self.add_report_builder(ExecutionHistoricReportBuilder(self.execution_historic, fn))
             
             fn = self.get_path("report_summary_scenario_failed.txt")
-            self.add_report_builder(SummaryScenarioFailedReportBuilder(fn))
+            self.add_report_builder(SummaryScenarioReportBuilder(fn, exclude_statuses=['Passed']))
             
             fn = self.get_path("report_summary_scenario_failed_by_category.txt")
-            self.add_report_builder(SummaryScenarioByCategoryReportBuilder(fn))
+            self.add_report_builder(SummaryScenarioByCategoryReportBuilder(fn, exclude_categories=['Always Success']))
             
             fn = self.get_path("report_summary_scenario_all.txt")
             self.add_report_builder(SummaryScenarioReportBuilder(fn))
@@ -77,15 +80,17 @@ class ReportManager(BaseReport):
             self.add_report_builder(FailureReportBuilder(fn))
             
             fn = self.get_path("report_short_scenario_failed.txt")
-            self.add_report_builder(ShortScenarioFailedReportBuilder(fn))
+            self.add_report_builder(ShortScenarioReportBuilder(fn, exclude_statuses=['Passed']))
             
-            # fn = self.get_path("report_detailed_scenario_failed.txt")
-            fn = self.get_path("report_detailed_scenario_failed.xml")
-            self.add_report_builder(DetailedScenarioFailedReportBuilder(fn))
+            fn = self.get_path(Config.campaign_manager_import_report_name)  # @UndefinedVariable
+            self.add_report_builder(DetailedScenarioReportBuilder(fn))
             
             fn = self.get_path("report_summary.txt")
             self.add_report_builder(SummaryReportBuilder(fn))
         
+            fn = self.get_path("report_summary_by_category.txt")
+            self.add_report_builder(SummaryByCategoryReportBuilder(fn))
+        
     @property
     def __feature_reports(self):
         return self.children_reports("feature")
@@ -234,64 +239,73 @@ class ReportManager(BaseReport):
             # Define scenario category
             category = None
             if cls._get_test_server_client().is_available:
-                # Get scenario execution statuses
-                scenario_name = f"{scenario.filename} at l.{scenario.line}"
-                sce_hist = cls._get_test_server_client().get_scenario_history(scenario_name=scenario_name, size=29)
-                statuses = [s['status'] for s in reversed(sce_hist[0]['statuses'])] if sce_hist else []
-                statuses.append(status)
-                
-                # Get scenario status sequences
-                passed_sequences = []
-                is_failed_relevant = None
-                for status in statuses:
-                    if status == 'Passed':
-                        passed = True
-                    elif status.startswith("Failed"):
-                        passed = False
-                        if status == "Failed":
-                            is_failed_relevant = True
-                        elif is_failed_relevant is None:
-                            is_failed_relevant = False
-                    else:
-                        continue
-                    
-                    if len(passed_sequences) == 0 or passed != passed_sequences[-1][0]:
-                        passed_sequences.append([passed, 1])
-                    else:
-                        passed_sequences[-1][1] += 1
-                
-                # Compute category
-                if passed_sequences:
-                    nb_exec = sum([x[1] for x in passed_sequences])
-                    last_passed, last_nb_times = passed_sequences[-1]
-                    if len(passed_sequences) == 1:
-                        if last_passed:
-                            category = f'Always Success ({last_nb_times})'
-                        elif is_failed_relevant:
-                            category = f'Always Failed ({last_nb_times})'
-                        else:
-                            category = f'Always Not Relevant ({last_nb_times})'
-                    elif last_passed and len(passed_sequences) in [2, 3]:
-                        category = f'Fixed ({last_nb_times} success / {nb_exec})'
-                    elif len(passed_sequences) > 2:
-                        nb_fail = sum([x[1] for x in passed_sequences if not x[0]])
-                        if is_failed_relevant:
-                            category = f'Random ({nb_fail} fails / {nb_exec})'
-                        else:
-                            category = f'Random but Not Relevant ({nb_fail} fails / {nb_exec})'
-                    elif not last_passed:
-                        if is_failed_relevant:
-                            category = f'Regression ({last_nb_times} fails / {nb_exec})'
-                        else:
-                            category = f'Regression but Not Relevant ({last_nb_times} fails / {nb_exec})'
-                    else:
-                        category = f'Unknown (unmanaged sequence: {passed_sequences})'
-                
-                logger.debug(f"Category of scenario '{scenario}': {category}  (computed from last statuses: {statuses})")
-                
-            cls._scenario_status_information_by_uid[scenario_uid] = [category, status, step_failed, step_nb]
+                category = cls._compute_category_validation(scenario, status)
+            
+            cls._scenario_status_information_by_uid[scenario_uid] = [category, status, step_failed, step_nb, 
+                                                                     SessionContext.instance().get_scenario_context(),
+                                                                     SessionContext.instance().get_scenario_context().get_step(step_nb-1) if step_nb is not None else None]
         
         return cls._scenario_status_information_by_uid[scenario_uid]
+    
+    @classmethod
+    def _compute_category_validation(cls, scenario, status):
+        res = None
+        
+        # Get scenario execution statuses
+        scenario_name = cls.format_scenario_short_description(scenario)
+        sce_hist = cls._get_test_server_client().get_scenario_history(scenario_name=scenario_name, size=29)
+        statuses = [s['status'] for s in reversed(sce_hist[0]['statuses'])] if sce_hist else []
+        statuses.append(status)
+        
+        # Get scenario status sequences
+        passed_sequences = []
+        is_failed_relevant = None
+        for status in statuses:
+            if status == 'Passed':
+                passed = True
+            elif status.startswith("Failed"):
+                passed = False
+                if status == "Failed":
+                    is_failed_relevant = True
+                elif is_failed_relevant is None:
+                    is_failed_relevant = False
+            else:
+                continue
+            
+            if len(passed_sequences) == 0 or passed != passed_sequences[-1][0]:
+                passed_sequences.append([passed, 1])
+            else:
+                passed_sequences[-1][1] += 1
+        
+        # Compute category
+        if passed_sequences:
+            nb_exec = sum([x[1] for x in passed_sequences])
+            last_passed, last_nb_times = passed_sequences[-1]
+            if len(passed_sequences) == 1:
+                if last_passed:
+                    res = f'Always Success ({last_nb_times})'
+                elif is_failed_relevant:
+                    res = f'Always Failed ({last_nb_times})'
+                else:
+                    res = f'Always Not Relevant ({last_nb_times})'
+            elif last_passed and len(passed_sequences) in [2, 3]:
+                res = f'Fixed ({last_nb_times} success / {nb_exec})'
+            elif len(passed_sequences) > 2:
+                nb_fail = sum([x[1] for x in passed_sequences if not x[0]])
+                if is_failed_relevant:
+                    res = f'Random ({nb_fail} fails / {nb_exec})'
+                else:
+                    res = f'Random but Not Relevant ({nb_fail} fails / {nb_exec})'
+            elif not last_passed:
+                if is_failed_relevant:
+                    res = f'Regression ({last_nb_times} fails / {nb_exec})'
+                else:
+                    res = f'Regression but Not Relevant ({last_nb_times} fails / {nb_exec})'
+            else:
+                res = f'Unknown (unmanaged sequence: {passed_sequences})'
+        
+        logger.debug(f"Category of scenario '{scenario}': {res}  (computed from last statuses: {statuses})")
+        return res
         
     @classmethod
     def get_step_failed_info(cls, scenario):
@@ -339,4 +353,72 @@ class ReportManager(BaseReport):
                 return cls.get_step_error_message(step)
         return None
     
+    @classmethod
+    def format_context_period(cls, context, format_precision_nsec=None, dt_ref=None, use_compact_format=default_value):
+        dt_start, dt_end = context.start_datetime, context.end_datetime
+        
+        # Prepare format of start datetime
+        if format_precision_nsec is not None:
+            dt_start = DateTime.truncate_datetime(dt_start, precision_nanoseconds=format_precision_nsec)
+        dt_format_start = cls._get_datetime_format_compared_to_reference(dt_start, dt_ref, use_compact_format=use_compact_format)
+        
+        # Prepare format of end datetime
+        if dt_end is not None:
+            if format_precision_nsec is not None:
+                dt_end = DateTime.truncate_datetime(dt_end, precision_nanoseconds=format_precision_nsec)
+            dt_format_end = cls._get_datetime_format_compared_to_reference(dt_end, dt_start, use_compact_format=use_compact_format)
+            if len(dt_format_end) > len(dt_format_start):
+                dt_format_start = dt_format_end
+        else:
+            dt_format_end = None
+        
+        # Format datetimes
+        start_txt = DateTime.datetime_2_str(dt_start, dt_format=dt_format_start)
+        end_txt = DateTime.datetime_2_str(dt_end, dt_format=dt_format_end) if dt_end is not None else ''
+        
+        # Truncate formatted datetimes if needed
+        if format_precision_nsec is not None:
+            trunc_len = len(f'{int(format_precision_nsec)}') - 4
+            if trunc_len > 0:
+                start_txt = start_txt[:-trunc_len-1]+'Z' if start_txt.endswith('Z') else start_txt[:-trunc_len]
+                if len(end_txt) > 0:
+                    end_txt = end_txt[:-trunc_len-1]+'Z' if end_txt.endswith('Z') else end_txt[:-trunc_len]
+        
+        return f"[{start_txt} - {end_txt}]"
+    
+    @classmethod
+    def _get_datetime_format_compared_to_reference(cls, dt, dt_ref=None, use_compact_format=default_value):
+        if use_compact_format is default_value:
+            use_compact_format = Config.report_compact_datetime_period  # @UndefinedVariable
+        
+        if dt_ref is None:
+            return FORMAT_DATETIME_ISO
+        
+        if not use_compact_format or dt.date() != dt_ref.date():
+            return FORMAT_DATETIME_ISO
+        elif dt.hour != dt_ref.hour:
+            return FORMAT_TIME_ISO
+        elif dt.minute != dt_ref.minute:
+            return '%M:%S.%f'
+        elif dt.second != dt_ref.second:
+            return '%S.%f'
+        else:
+            return '.%f'
+    
+    @classmethod
+    def format_scenario_short_description(cls, scenario):
+        return f"{scenario.filename} at l.{scenario.line}"
+    
+    @classmethod
+    def format_step_short_description(cls, step, step_number, step_context=None, dt_ref=None, has_failed=undefined_argument):
+        if step:
+            if step_context and step_context.start_datetime is not None:
+                return f"step {step_number} (l.{step.line} on {cls.format_context_period(step_context, dt_ref=dt_ref)})"
+            else:
+                return f"step {step_number} (l.{step.line})"
+        elif has_failed is not undefined_argument and has_failed:
+            return "step ? (missing step implementation ?)"
+        else:
+            return None
+            
     
