@@ -18,6 +18,7 @@ import behave
 from holado.common.context.session_context import SessionContext
 from holado_core.common.exceptions.technical_exception import TechnicalException
 from holado_multitask.multitasking.multitask_manager import MultitaskManager
+import time
 
 logger = logging.getLogger(__name__)
 
@@ -390,6 +391,8 @@ class BehaveManager(object):
                                       lambda: context_exists(MultitaskManager.get_process_id(), thread_uid))
         wait_context.redo_until(True)
         wait_context.execute()
+        # Note: previous waiting mechanism is sometimes to rapid to execute steps just after
+        time.sleep(0.01)
         
         ### Other tried implementation:
         
