@@ -13,6 +13,11 @@
 from flask.views import MethodView
 from holado.common.context.session_context import SessionContext
 from holado.common.handlers.undefined import default_value
+import logging
+
+
+logger = logging.getLogger(__name__)
+
 
 def _get_session_context():
     return SessionContext.instance()
@@ -21,95 +26,119 @@ def _get_session_context():
 class StatusView(MethodView):
     
     def get(self, name, all_=False):
-        cont = _get_session_context().docker_client.get_container(name, all_=all_, reset_if_removed=False)
-        if cont:
-            return cont.status
-        else:
-            return
+        try:
+            cont = _get_session_context().docker_client.get_container(name, all_=all_, reset_if_removed=False)
+            if cont:
+                return cont.status
+            else:
+                return
+        except Exception as exc:
+            logger.exception(f"Failed to get status of container '{name}'")
+            return str(exc), 500
 
 class HealthStatusView(MethodView):
     
     def get(self, name, all_=False):
-        cont = _get_session_context().docker_client.get_container(name, all_=all_, reset_if_removed=False)
-        if cont:
-            return cont.health_status
-        else:
-            return
+        try:
+            cont = _get_session_context().docker_client.get_container(name, all_=all_, reset_if_removed=False)
+            if cont:
+                return cont.health_status
+            else:
+                return
+        except Exception as exc:
+            logger.exception(f"Failed to get health status of container '{name}'")
+            return str(exc), 500
 
 
 
 class RestartView(MethodView):
     
     def put(self, name, wait_started=True, timeout=default_value, body:dict=None):
-        kwargs = body if body is not None else {}
-        
-        if not _get_session_context().docker_client.has_container(name):
-            return f"Container '{name}' doesn't exist", 410
-        
         try:
+            kwargs = body if body is not None else {}
+            
+            if not _get_session_context().docker_client.has_container(name):
+                return f"Container '{name}' doesn't exist", 410
+            
             return _get_session_context().docker_client.restart_container(name, wait_started=wait_started, timeout=timeout, **kwargs)
         except Exception as exc:
+            logger.exception(f"Failed to restart container '{name}'")
             return str(exc), 500
 
 class StartView(MethodView):
     
     def put(self, name, wait_started=True, timeout=default_value, body:dict=None):
-        kwargs = body if body is not None else {}
-        
-        if not _get_session_context().docker_client.has_container(name, all_=True):
-            return f"Container '{name}' doesn't exist", 410
-        
         try:
+            kwargs = body if body is not None else {}
+            
+            if not _get_session_context().docker_client.has_container(name, all_=True):
+                return f"Container '{name}' doesn't exist", 410
+            
             return _get_session_context().docker_client.start_container(name, wait_started=wait_started, timeout=timeout, **kwargs)
         except Exception as exc:
+            logger.exception(f"Failed to start container '{name}'")
             return str(exc), 500
 
 class StopView(MethodView):
     
     def put(self, name, wait_stopped=True, timeout=default_value, body:dict=None):
-        kwargs = body if body is not None else {}
-        
-        if not _get_session_context().docker_client.has_container(name):
-            return f"Container '{name}' doesn't exist", 410
-        
         try:
+            kwargs = body if body is not None else {}
+            
+            if not _get_session_context().docker_client.has_container(name):
+                return f"Container '{name}' doesn't exist", 410
+            
             return _get_session_context().docker_client.stop_container(name, wait_stopped=wait_stopped, timeout=timeout, **kwargs)
         except Exception as exc:
+            logger.exception(f"Failed to stop container '{name}'")
             return str(exc), 500
 
 class WaitView(MethodView):
     
     def put(self, name, timeout=default_value, body:dict=None):
-        kwargs = body if body is not None else {}
-        
-        if not _get_session_context().docker_client.has_container(name):
-            return f"Container '{name}' doesn't exist", 410
-        if timeout is not default_value:
-            kwargs['timeout'] = timeout
-        
         try:
+            kwargs = body if body is not None else {}
+            
+            if not _get_session_context().docker_client.has_container(name):
+                return f"Container '{name}' doesn't exist", 410
+            if timeout is not default_value:
+                kwargs['timeout'] = timeout
+            
             return _get_session_context().docker_client.get_container(name).wait(**kwargs)
         except Exception as exc:
+            logger.exception(f"Failed to wait container '{name}'")
             return str(exc), 500
 
 class AwaitStartedView(MethodView):
     
     def put(self, name, timeout=default_value):
-        _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
-        res = _get_session_context().docker_client.get_container(name).await_started(timeout=timeout)()
-        return res
+        try:
+            _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
+            return _get_session_context().docker_client.get_container(name).await_started(timeout=timeout)()
+        except Exception as exc:
+            logger.exception(f"Failed to await container '{name}' is started")
+            return str(exc), 500
     
 class AwaitStatusView(MethodView):
     
     def put(self, name, status, timeout=default_value):
-        _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
-        res = _get_session_context().docker_client.get_container(name).await_status(status, timeout=timeout)()
-        return res
+        try:
+            _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
+            return _get_session_context().docker_client.get_container(name).await_status(status, timeout=timeout)()
+        except Exception as exc:
+            logger.exception(f"Failed to await container '{name}' has status '{status}'")
+            return str(exc), 500
     
 class AwaitHealthStatusView(MethodView):
     
     def put(self, name, status, timeout=default_value):
-        _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
-        res = _get_session_context().docker_client.get_container(name).await_health_status(status, timeout=timeout)()
-        return res
-    
+        try:
+            _get_session_context().docker_client.await_container_exists(name, timeout=timeout)
+            return _get_session_context().docker_client.get_container(name).await_health_status(status, timeout=timeout)()
+        except Exception as exc:
+            logger.exception(f"Failed to await container '{name}' has health status '{status}'")
+            return str(exc), 500
+
+
+
+
