@@ -2,7 +2,7 @@
 
 
 # Import generic environment methods
-from test_testing_solution_ksc.behave_environment import *
+from holado_test.behave.behave_environment import *
 
 # Define project specific environment methods
 
